<?php

namespace Drupal\edit_plus_workspaces\ParamConverter;

use Drupal\Core\Messenger\MessengerInterface;
use Drupal\workspaces\WorkspaceAssociationInterface;
use Drupal\workspaces\WorkspaceManagerInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\edit_plus\EditPlusTempstoreRepository;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\edit_plus\ParamConverter\EntityConverter;
use Drupal\Core\ParamConverter\ParamConverterInterface;

/**
 * Class WorkspacesEntityConverter
 */
class WorkspacesEntityConverter extends EntityConverter {

  public function __construct(
    protected ParamConverterInterface $inner,
    EntityTypeManagerInterface $entity_type_manager,
    EntityRepositoryInterface $entity_repository,
    protected EditPlusTempstoreRepository $editPlusTempstoreRepository,
    protected MessengerInterface $messenger,
    protected WorkspaceManagerInterface $workspaceManager,
  ) {
    parent::__construct($inner, $entity_type_manager, $entity_repository, $editPlusTempstoreRepository, $messenger);
  }

  /**
   * Override the convert method.
   */
  public function convert($value, $definition, $name, array $defaults) {
    if ($this->workspaceManager->hasActiveWorkspace()) {
      $workspace = $this->workspaceManager->getActiveWorkspace();
      // I don't seem to be able to inject the association service.
      $tracked_entities = \Drupal::service('workspaces.association')->getTrackedEntities($workspace->id());
      if (!empty($tracked_entities[$name])) {
        $tracked_entities[$name] = array_flip($tracked_entities[$name]);
        if (!empty($tracked_entities[$name][$value])) {
          return parent::convert($value, $definition, $name, $defaults);
        }
      }
    }

    return $this->inner->convert($value, $definition, $name, $defaults);

  }
}
